"""Generated GraphQL API models.

Files in this package are generated from live Atlassian GraphQL Gateway (AGG) schema
introspection. Do not edit by hand; use the corresponding generator scripts in
`python/tools/`.
"""

